import java.io.File;
import java.text.DecimalFormat;
import java.util.HashMap;
import java.util.Set;
import edu.njust.cs.bio.protein.PSSM;
import edu.njust.cs.bio.util.FASTAFileParser;
//Generate PSSMs of protein sequences.
// 
public class GenerateTotalPSSM {	
	private static String dataBasePath = "";  
	private static String fileName = ""; 
	
public static void main(String []args){
		
		if (args.length != 2){
			System.out.println("Usage:   java GenerateTotalPSSM dataBasePath  fileName");
			System.out.println("e.g.:   java GenerateTotalPSSM ../db126   db_126_protein.fasta");
			System.out.println("e.g.:   java GenerateTotalPSSM ../db120   db_120_protein.fasta");
			System.exit(-1);
		}
		
		dataBasePath = args[0];
		fileName = args[1];
		
		//Step 1. Clear original files in totalPSSMs/
		File dir = new File(dataBasePath +"/totalPSSMs");
		File[] files = dir.listFiles();
		if (files != null) {
			for (int i = 0; i < files.length; i++)
				files[i].delete();
		}
		
		//Step 2. Generate PSSMs		
		generatePSSMs(dataBasePath + "/" + fileName);
		
	}

	private static void generatePSSMs(String proteinFileName) {

		HashMap<String, String> proteinSeqs = FASTAFileParser
				.parseProteinSeqs(proteinFileName);
		Set<String> keys = proteinSeqs.keySet();
		Object[] keysArray = keys.toArray();

		for (int i = 0; i < keysArray.length; i++) {
			String proteinCode = (String) keysArray[i];
			String proteinSeq = (String) proteinSeqs.get(keysArray[i]);
			System.out.println(proteinCode);
			System.out.println(proteinSeq);

			String contents = "\n\n...Processing Protein: [" + proteinCode
					+ "]...";
			System.out.println(contents);
			generatePSSM(proteinCode, proteinSeq);
		}
	}
	private static void generatePSSM(String proteinCode, String proteinSeq) {
		DecimalFormat df = new DecimalFormat();
		df.applyPattern("000");
		// 
		String contents = "...... 1. generate PSSM which is stored at ./temp/normalizedPSSM";
		System.out.println(contents);

		PSSM pssm = new PSSM(proteinCode, proteinSeq);
		pssm.computePSSM();
		// 
		contents = "...... 2. ȡPSSM Matrix";
		System.out.println(contents);
		double[][] pssmMatrix = pssm.getNormalizedPSSM();
		
		int index = proteinCode.indexOf(" ");
		String proteinCodeShort = proteinCode.substring(0, index);
		String fileName = proteinCodeShort + ".mat";
		PSSM.saveMatrix(dataBasePath + "\\totalPSSMs", fileName, pssmMatrix);
	}
	
}
