
import java.util.*;
import edu.njust.cs.bio.protein.PSSM;
import edu.njust.cs.bio.protein.TMHSplitter;
import edu.njust.cs.bio.util.FASTAFileParser;
import edu.njust.cs.bio.util.Filter;
import edu.njust.cs.util.FileObject;

public class TransPredictor {

	private String dataBase = null;
	private double threshold = 0.475;
	private double step = 0.02;
    //DB126   0.475  0.02
	public TransPredictor() {
	}

	public TransPredictor(String db) {
		dataBase = db;
	}
	public void extractTMHs() {
			
		HashMap<String, String> proteinSeqs = FASTAFileParser.parseProteinSeqs(dataBase+"/current/testingProtein.fasta");
		Set<String> keys = proteinSeqs.keySet();
		Object [] keysArray = keys.toArray();
		Arrays.sort(keysArray);
		
		StringBuffer initialTMHBuf = new StringBuffer();
		StringBuffer finalTMHBuf = new StringBuffer();
		
		for (int k = 0; k  < keysArray.length; k ++) {	
			
			String proteinCode = (String )keysArray[k];
			int index = proteinCode.indexOf(" ");
			String proteinCodeShort = proteinCode.substring(0, index);
			initialTMHBuf.append(">" + proteinCodeShort);
			finalTMHBuf.append(">" + proteinCodeShort);		

			String som_fileName = dataBase + "/result/sompnn_propensity/" + proteinCodeShort + "_propensity.mat";
			double [] som_propensity = PSSM.readPropensity(som_fileName);
		
			double [] propensity = new double [som_propensity.length];
	
			for (int i = 0 ; i< som_propensity.length ; i ++){
				 propensity[i] =  som_propensity[i]; //SOMPNN
			}
			propensity = Filter.modifiedMedianFilter(propensity, 5);
			//get initial TMHs
			Vector<int []> initialTMHs = getInitialTMHs(propensity, threshold);
			
			for (int i = 0; i < initialTMHs.size(); i++) {				
				int tmh_begin = initialTMHs.elementAt(i)[0]+1;
				int tmh_end = initialTMHs.elementAt(i)[1] + 1;
				initialTMHBuf.append("  " + tmh_begin + "-" + tmh_end );
			}
			initialTMHBuf.append("\n");
			
			//get final TMHs			
			Vector<int []> finalTMHs = TMHSplitter.getFinalTMHs(
					initialTMHs, propensity, threshold , step);
			if(finalTMHs.size()>0){
			for (int i = 0; i < finalTMHs.size(); i++) {				
				int tmh_begin = finalTMHs.elementAt(i)[0] ;
				if(tmh_begin == 0 )
					tmh_begin = 1;
				int tmh_end = finalTMHs.elementAt(i)[1] +1  ;
				if(tmh_end > propensity.length )
					tmh_end =  propensity.length ;
				finalTMHBuf.append("  " + tmh_begin + "-" + tmh_end );
			}
			}else{
				finalTMHBuf.append("  NONE" );
			}
			finalTMHBuf.append("\n");
		}
			

		FileObject fo = new FileObject();
		fo.setContents(initialTMHBuf.toString().trim().getBytes());
		System.out.println(initialTMHBuf.toString().trim());		
		String fileName = "predicted_SOMPNN_initial.fasta";
		fo.writeToFile( dataBase + "/result", fileName);
		
		fo.setContents(finalTMHBuf.toString().trim().getBytes());
		System.out.println(finalTMHBuf.toString().trim());		
		fileName = "predicted_SOMPNN.fasta";
		fo.writeToFile( dataBase + "/result", fileName);
	}

	private Vector<int []> getInitialTMHs(double[] propensity,
			double threshold) {
		// 1. ȡóʼTMH
		Vector<int []> initialTMHs = new Vector<int []>();
		double pre_score = -1;
		int begin_TMH = -1;
		int end_TMH = -1;
		for (int i = 0; i < propensity.length; i++) {
			
			double score = propensity[i];
			if (pre_score < threshold && score >= threshold) {
				begin_TMH = i;
			}
			if (pre_score >= threshold && score < threshold) {
				end_TMH = i - 1;
				if (begin_TMH == -1) {
					int [] aTMH = new int [2];
					aTMH[0] = 0; // begin TMH
					aTMH[1] = end_TMH; // end TMH
					initialTMHs.add(aTMH);
				} else {
					int [] aTMH = new int [2];
					aTMH[0] =  begin_TMH; // begin TMH
					aTMH[1] = end_TMH; // end TMH
					initialTMHs.add(aTMH);
				}
			}
			
			if(i == propensity.length -1){  // the last Animo Acid
				if(score > threshold){
					end_TMH = propensity.length -1;
					
					if (begin_TMH == -1) {
						int [] aTMH = new int [2];
						aTMH[0] = 0; // begin TMH
						aTMH[1] = end_TMH; // end TMH
						initialTMHs.add(aTMH);
					} else {
						int [] aTMH = new int [2];
						aTMH[0] =  begin_TMH; // begin TMH
						aTMH[1] = end_TMH; // end TMH
						initialTMHs.add(aTMH);
					}
				}
			}
			
			pre_score = propensity[i];
		}

		// System.out.println(initialTMHs.size());
		// ȥ<5TMH
		Vector<int []> temp_initialTMHs = new Vector<int []>();
		for (int i = 0; i < initialTMHs.size(); i++) {
			temp_initialTMHs.addElement(initialTMHs.elementAt(i));
		}
		initialTMHs.removeAllElements();
		for (int i = 0; i < temp_initialTMHs.size(); i++) {
			int temp_begin = temp_initialTMHs.elementAt(i)[0];
			int temp_end = temp_initialTMHs.elementAt(i)[1];

			if (Math.abs(temp_begin - temp_end) >7) {
				initialTMHs.addElement(temp_initialTMHs.elementAt(i));
			}
		}
		return initialTMHs;
	}

	public static void main(String[] args) {
		if (args.length != 1){
			System.out.println("Usage:   java TransPredictor dataBasePath");
			System.out.println("e.g.:   java TransPredictor ../db126");
		}
		String dataBase = args[0];
		TransPredictor tp = new TransPredictor(dataBase);
		tp.extractTMHs();
	}
}