package edu.njust.cs.bio.performance;

import java.text.DecimalFormat;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Set;
import java.util.Vector;

import edu.njust.cs.bio.util.FASTAFileParser;
import edu.njust.cs.bio.util.ProteinSeqAnalysis;
import edu.njust.cs.util.FileObject;

public class PerformanceAnalysis {
	
	private static DecimalFormat df = new DecimalFormat();
	static{
	df.applyPattern("#0.00");
	}
	public static void sortPredictionResult(String fileName) {
		HashMap<String, String> proteinSeqs = FASTAFileParser
				.parseProteinSeqs(fileName);

		Set<String> keys = proteinSeqs.keySet();
		Object[] keysArray = keys.toArray();

		Arrays.sort(keysArray);

		StringBuffer buf = new StringBuffer();
		for (int i = 0; i < keysArray.length; i++) {
			//System.out.println(i);
			buf.append(">" + keysArray[i]);
			buf.append("\n");
		}

		FileObject fo = new FileObject();
		fo.setContents(buf.toString().trim().getBytes());
		fo.writeToFile("./", fileName);

	}

	public static Object[] getTestingProteinCodeShort(
			HashMap<String, Vector<int[]>> testingResult) {
		Set<String> keys = testingResult.keySet();
		Object[] testingProteinCodeShort = keys.toArray();
		Arrays.sort(testingProteinCodeShort);
		return testingProteinCodeShort;
	}

	public static int getTotalTMHs(HashMap<String, Vector<int[]>> testingResult,
			HashMap<String, Vector<int[]>> observedResult) {
		Set<String> keys = testingResult.keySet();
		Object[] testingProteinCodeShort = keys.toArray();
		
		int totalNumOfObservedTMHs = 0;
		for (int i = 0; i < testingProteinCodeShort.length; i++) {
			Vector<int[]> tmhs = observedResult.get(testingProteinCodeShort[i]);
		//	System.out.println(testingProteinCodeShort[i]);
			
			totalNumOfObservedTMHs += tmhs.size();
		}
		return totalNumOfObservedTMHs;
	}

	private static int getCorrectPredictedTMHsOfGivenProtein(
			Vector<int[]> testing, Vector<int[]> observed) {
		int correctedPredictedNum = 0;
		for (int i = 0; i < observed.size(); i++) {
			int[] observedTMH = observed.elementAt(i);
			int observed_a = observedTMH[0];
			int observed_b = observedTMH[1];
			int overLappedResidue_max = Integer.MIN_VALUE;
			//System.out.println(testing);
			for (int j = 0; j < testing.size(); j++) {
				int[] testingTMH = testing.elementAt(j);
				int predicted_a = testingTMH[0];
				int predicted_b = testingTMH[1];

				int overLappedResidue = getNumOfOverlappedResidue(predicted_a,
						predicted_b, observed_a, observed_b);
				if (overLappedResidue > overLappedResidue_max)
					overLappedResidue_max = overLappedResidue;
			}

			int realLength = observed_b - observed_a + 1;
			if (realLength > 9) {
				if (overLappedResidue_max >= 9)
					correctedPredictedNum++;
			} else {
				if (overLappedResidue_max > realLength / 2)
					correctedPredictedNum++;
			}
		}

		return correctedPredictedNum;
	}

	public static int getCorrectPredictedTMHs(HashMap<String, Vector<int[]>> testingResult,
			HashMap<String, Vector<int[]>> observedResult) {
		Set<String> keys = testingResult.keySet();
		Object[] testingProteinCodeShort = keys.toArray();
		
		int correctedPredictedTMHs = 0;
		for (int i = 0; i < testingProteinCodeShort.length; i++) {
			Vector<int[]> testing = testingResult
					.get(testingProteinCodeShort[i]);
			Vector<int[]> observed = observedResult
					.get(testingProteinCodeShort[i]);
			//System.out.println(testingProteinCodeShort[i]);
			correctedPredictedTMHs += getCorrectPredictedTMHsOfGivenProtein(
					testing, observed);
			
		}
		return correctedPredictedTMHs;
	}
	
	public static String getN_C_Score(	HashMap<String, Vector<int[]>> testingResult,
			HashMap<String, Vector<int[]>> observedResult, char type ){
		Set<String> keys = testingResult.keySet();
		Object[] testingProteinCodeShort = keys.toArray();
		if(type!='N' &&type!='C')
			System.out.println("FATAL ERROR:   Score Type Should Be 'N' or 'C'. ");

		Vector <float []>  scoreVector = new Vector<float []>();
		
		for (int i = 0; i < testingProteinCodeShort.length; i++) {
			Vector<int[]> testing = testingResult
					.get(testingProteinCodeShort[i]);
			Vector<int[]> observed = observedResult
					.get(testingProteinCodeShort[i]);
			float [] score =  getN_C_ScoreOfGivenProtein(testing, observed, type,(String)testingProteinCodeShort[i]);
			scoreVector.addElement(score);
		}
		
		//ֵ
		float totalScore = (float) 0.0;
		int totalHelixNum = 0;
		for(int i = 0; i <scoreVector.size(); i++){
			float [] scores = scoreVector.elementAt(i);
			for(int j = 0; j < scores.length; j ++){
				totalHelixNum ++;
				totalScore += scores[j];
				//System.out.println( scores[j]);
			}
		}
		
		float mean = totalScore/ (float)totalHelixNum;
		
		//׼ƽ͵ƽƽΪһºȵһֵָꡣ
		float error = (float) 0.0;
		for(int i = 0; i <scoreVector.size(); i++){
			float [] scores = scoreVector.elementAt(i);
			for(int j = 0; j < scores.length; j ++){
				error += Math.pow(mean - scores[j], 2);
			}
		}
		
		float deviation = (float)Math.sqrt(error/(float)(totalHelixNum-1));
		
		return df.format(mean)+"+/-" + df.format(deviation) ;
	}
	
	
	private static float [] getN_C_ScoreOfGivenProtein(Vector<int[]> testing, Vector<int[]> observed, char type,String testingProteinCodeShort){
		float [] score = new float[observed.size()];
		for (int i = 0; i < observed.size(); i++) {
			int[] observedTMH = observed.elementAt(i);
			int observed_a = observedTMH[0];
			int observed_b = observedTMH[1];
			int overLappedResidue_max = Integer.MIN_VALUE;
			
            int theMostMatchIndex = -1;			
			for (int j = 0; j < testing.size(); j++) {
				int[] testingTMH = testing.elementAt(j);
				int testing_a = testingTMH[0];
				int testing_b = testingTMH[1];

				int overLappedResidue = getNumOfOverlappedResidue(testing_a,
						testing_b, observed_a, observed_b);
				if (overLappedResidue > overLappedResidue_max){
					overLappedResidue_max = overLappedResidue;
					theMostMatchIndex = j;
				}
			}
			
			if(overLappedResidue_max<=0){
				score[i] = (float)0.5 *Math.abs(observed_b - observed_a  + 1 );
				//System.out.println(overLappedResidue_max);
				if(score[i] >5){
					//System.out.println(testingProteinCodeShort+ "   "+observed_a+ "-"+ observed_b  + "   "+score[i] );
				}
			}
			else{
				int [] mostMatchTMH = testing.elementAt(theMostMatchIndex);
				int mostMatch_a = mostMatchTMH[0];
				int mostMatch_b = mostMatchTMH[1];
				if(type == 'N')
					score[i] = Math.abs(observed_a - mostMatch_a);
				else
					score[i] = Math.abs(observed_b - mostMatch_b);
				
		    	if(score[i] >5){
					//System.out.println(testingProteinCodeShort+ "   "+observed_a+ "-"+ observed_b  + "   "+score[i] );
		    	}

			}
		}
		
		
		return score;
	}
	
	public static int getCorrectPredictedProteins(
		HashMap<String, Vector<int[]>> testingResult,
		HashMap<String, Vector<int[]>> observedResult, boolean display) {
		Set<String> keys = testingResult.keySet();
		Object[] testingProteinCodeShort = keys.toArray();
		
		int correctedPredictedProteins = 0;
		if (display)
			System.out.println("InCorrectly Predicted Protein:   ");
		for (int i = 0; i < testingProteinCodeShort.length; i++) {
			Vector<int[]> testing = testingResult
					.get(testingProteinCodeShort[i]);
			Vector<int[]> observed = observedResult
					.get(testingProteinCodeShort[i]);

			if (isTheProteinCorrectlyPredicted(testing, observed)) {
				correctedPredictedProteins++;
			} else {
				if (display) {
					System.out.print(">" + testingProteinCodeShort[i]);
					for (int k = 0; k < testing.size(); k++) {
						int[] t = testing.elementAt(k);
						System.out.print("  " + t[0] + "-" + t[1]);
					}
					System.out.println();
				}
			}

		}
		return correctedPredictedProteins;
	}

	private static boolean isTheProteinCorrectlyPredicted(
			Vector<int[]> testing, Vector<int[]> observed) {
		if (testing.size() != observed.size()) // ԤTMH
			return false;

		int correctedPredictedTMHs = 0;

		for (int i = 0; i < observed.size(); i++) {
			int[] observedTMH = observed.elementAt(i);
			int observed_a = observedTMH[0];
			int observed_b = observedTMH[1];

			int[] testingTMH = testing.elementAt(i);
			int predicted_a = testingTMH[0];
			int predicted_b = testingTMH[1];

			int realLength = observed_b - observed_a + 1;
			int overLappedResidue = getNumOfOverlappedResidue(predicted_a,
					predicted_b, observed_a, observed_b);

			if (realLength > 9) {				
				if (overLappedResidue >= 9)
					correctedPredictedTMHs++;				
			} else {
				if (overLappedResidue > realLength / 2)
					correctedPredictedTMHs++;
			}
		}

		if (correctedPredictedTMHs == observed.size())
			return true;
		else
			return false;
		
		
		
	}

	private static int getNumOfOverlappedResidue(int predicted_a,
			int predicted_b, int observed_a, int observed_b) {
		int overLappedResidue = 0;
		int min = Math.min(Math.min(Math.min(observed_a, observed_b),
				predicted_a), predicted_b);
		int max = Math.max(Math.max(Math.max(observed_a, observed_b),
				predicted_a), predicted_b);

		int temp = 0;
		for (int k = min; k <= max; k++) {
			if (k >= observed_a && k <= observed_b && k >= predicted_a
					&& k <= predicted_b)
				temp++;
		}
		if (temp > overLappedResidue)
			overLappedResidue = temp;
		return overLappedResidue;
	}
	
	
	
	public static String getRMSD(
			HashMap<String, Vector<int[]>> testingResult,
			HashMap<String, Vector<int[]>> observedResult, HashMap<String, Integer> observedSeqLen){
		Set<String> keys = testingResult.keySet();
		Object[] testingProteinCodeShort = keys.toArray();
		
		Vector <Float>  scoreVector = new Vector<Float>();
		
		for (int i = 0; i < testingProteinCodeShort.length; i++) {
			Vector<int[]> testing = testingResult
					.get(testingProteinCodeShort[i]);
			Vector<int[]> observed = observedResult
					.get(testingProteinCodeShort[i]);
			
			int length  = observedSeqLen.get(testingProteinCodeShort[i]).intValue();		
			
			//System.out.println(testingProteinCodeShort[i] +" " +length);
			Float score =  getRMSDOfGivenProtein(testing, observed, length);
			//System.out.println(score);
			
			scoreVector.addElement(score);
		}
		
		//ֵ
		float totalScore = (float) 0.0;
		for(int i = 0; i <scoreVector.size(); i++){
				totalScore += scoreVector.elementAt(i);
			//	System.out.print(scoreVector.elementAt(i)+ " ");
		
		}		
		
		float mean = totalScore/ (float)scoreVector.size();
		
		//׼ƽ͵ƽƽΪһºȵһֵָꡣ
		float error = (float) 0.0;
		for(int i = 0; i <scoreVector.size(); i++){
			error += Math.pow(mean - scoreVector.elementAt(i), 2);
		}
		
		float deviation = (float)Math.sqrt(error/(float)(scoreVector.size()-1));
		
		return df.format(mean)+"+/-" + df.format(deviation) ;
		
	}
	
	
	private static Float getRMSDOfGivenProtein(Vector<int[]> testing, Vector<int[]> observed, int seqLen){

		   float [] testingFlags = ProteinSeqAnalysis.getTMH_Flags(testing , seqLen);
	        //	   for(int i = 0; i < testingFlags.length; i ++)
			//   System.out.println(testingFlags[i]);
		   float [] observedFlags = ProteinSeqAnalysis.getTMH_Flags(observed , seqLen);
		   //   System.out.println(seqLen);
		   float fenzi = 0;
		   float fenmu = 0;
		   for(int i = 0; i< observedFlags.length; i ++){
			   fenzi =  fenzi + (float)Math.pow((float)testingFlags[i] - (float)observedFlags[i],2.0);
			   fenmu = fenmu + (float)Math.pow( (float)observedFlags[i],2.0);
		   }
		   fenzi = (float)Math.sqrt((float)fenzi);
		   fenmu = (float)Math.sqrt((float)fenmu);
		   
		//   System.out.println(fenzi  +"  " +fenmu);
		   
		  // fenmu = (float)Math.sqrt((float)observedFlags.length);
		   return fenzi/fenmu;
		
	}
	


}
