package edu.njust.cs.bio.protein;


import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.text.DecimalFormat;
import java.util.*;

import edu.njust.cs.util.FileObject;
import edu.njust.cs.util.SocketUtil;
import edu.njust.cs.util.StrUtil;

//Ttemporary files generalized by this code: 
//1. /temp/queryProteinSeq
//2. /temp/blastPSSM
//3. /temp/originalPSSM
//4. /temp/normalizedPSSM

public class PSSM {
	// Input
	private String proteinCode = null;

	private String proteinSeq = null;

	// Output:
	private int[][] originalPSSM = null;

	private double[][] normalizedPSSM = null;

	public PSSM(String proteinCode, String proteinSeq) {
		this.proteinCode = proteinCode;
		this.proteinSeq = proteinSeq;
	}

	public PSSM() {

	}

	public int[][] getOriginalPSSM() {
		return originalPSSM;
	}

	public double[][] getNormalizedPSSM() {
		return normalizedPSSM;
	}

	public String getProteinCode() {
		return proteinCode;
	}

	public String getProteinSeq() {
		return proteinSeq;
	}
	public static double []readPropensity(String fileName){
		Vector<Double> propensity = new Vector<Double>();
		
		BufferedReader in = null;
		try {			
			in = new BufferedReader(new FileReader(
					fileName));
		    String line = in.readLine();

			while (line != null && !line.trim().equals("")) {
				propensity.addElement(Double.parseDouble(line));
				line = in.readLine();
			}
		}catch(Exception e){
			e.printStackTrace();
		}finally{
				SocketUtil.close(in);
		}
		double [] p = new double[propensity.size()];
		for (int i = 0; i < propensity.size(); i++) {
			p[i] = propensity.elementAt(i);
		}
		return p;
	}
	public void computePSSM() {
		// Step 1. Write to a temp file
		String querySeqFileName = "queryProteinSeq";
		// System.out.println("Step 1. Write to a temp file: " +
		// querySeqFileName + "...");
		//int k = proteinCode.indexOf(" ");
		// String pc = proteinCode.substring(0, k);

		StringBuffer buf = new StringBuffer(">" + proteinCode);
		System.out.println(proteinCode);
		buf.append("\n");
		buf.append(proteinSeq);
		FileObject fu = new FileObject(buf.toString().getBytes());
		fu.writeToFile("./temp",querySeqFileName);

		// Step 2. Construct blast PSSM Matrix
		String blastPSSMMatrix = "blastPSSM";
		System.out.println("Step 2. Construct blast PSSM: " + blastPSSMMatrix
				+ " ...");
		try {
			System.out.print("    Begin run PSI-Blast, Waiting...\n");
			String cmd = "cmd /c  blastpgp -j 3 -i " + "./temp/"+querySeqFileName
					+ " -d ./proteindb/uniprot_sprot.fasta -Q "
					+ "./temp/"+blastPSSMMatrix;
			System.out.println("    " + cmd);

			Process process = Runtime.getRuntime().exec(cmd);

			// ---------δĿǴDOS׼ȡݣֹprocess---------
			StreamGobbler errorGobbler = new StreamGobbler(process
					.getErrorStream(), "Error");
			StreamGobbler outputGobbler = new StreamGobbler(process
					.getInputStream(), "Output");
			errorGobbler.start();
			outputGobbler.start();

			try {
				process.waitFor(); //
			} catch (Exception e) {
				e.printStackTrace();
			}
			// -------------------------------------------------------------------------------------------
			System.out.println(" PSI-Blast Finished!");
			System.out.println(" PSI-Blast exit code : " + process.exitValue());
			if (process.exitValue() == 0)
				System.out.println(" PSI-Blast Succeed!");
			else
				System.out.println(" PSI-Blast Failed!");

			try {
				process.destroy();
			} catch (Exception e) {
				e.printStackTrace();
			}

		} catch (Exception e) {
			e.printStackTrace();
		}

		// System.out.println("Step 3. Parsing blast PSSM and Saving original
		// PSSM ...");
		parseBlastPSSM("./temp/" + blastPSSMMatrix);

		// System.out.println("Step 4. Normalizing original PSSM and Saving
		// Normalized PSSM...");
		normalizeOriginalPSSM();

	}

	private void normalizeOriginalPSSM() {
		// System.out.println(" Normalizing PSSM ...!");
		// int [][]originalPSSM ={{-2, -3, -5, -4, -1, -4, -3, 0, -3, 1, 10, -4,
		// -4, -2, -3, -3, -2, -1, -3, -2 }};

		int row = originalPSSM.length;
		int col = originalPSSM[0].length;
		this.normalizedPSSM = new double[row][col];

		for (int i = 0; i < originalPSSM.length; i++) {
			// Calculate meanValue of the i-th line
			double meanValue = 0;
			for (int j = 0; j < originalPSSM[i].length; j++) {
				meanValue += originalPSSM[i][j];
			}
			meanValue = meanValue / 20;
			// Calculate fenmu
			double fenmu = 0;
			for (int j = 0; j < originalPSSM[i].length; j++) {
				fenmu = fenmu + (originalPSSM[i][j] - meanValue)
						* (originalPSSM[i][j] - meanValue);
			}
			fenmu = Math.sqrt(fenmu / 20);
			// Calculate normalized PSSM
			for (int j = 0; j < originalPSSM[i].length; j++) {
				double fenzi = originalPSSM[i][j] - meanValue;
				if (fenmu == 0) {
					normalizedPSSM[i][j] = 0;
				} else {
					normalizedPSSM[i][j] = fenzi / fenmu;
				}
			}
		}

		saveMatrix("./temp","normalizedPSSM", normalizedPSSM);
	}

	
	
	public static void saveMatrix(String path, String fileName, double[][] matrix) {

		// System.out.println(" Saving Normalized PSSM to: " +fileName);
		DecimalFormat df = new DecimalFormat();
		df.applyPattern("#0.000000");
		StringBuffer buf = new StringBuffer();
		for (int i = 0; i < matrix.length; i++) {
			for (int j = 0; j < matrix[i].length; j++) {
				buf.append(StrUtil.formatStrToFixLen(df.format(matrix[i][j]),
						10));
			}
			buf.append("\n");
		}
		// System.out.println(buf.toString());
		FileObject fo = new FileObject();
		String str = buf.toString();
		str = str.substring(0, str.length() -1 );
		fo.setContents(str.getBytes());
		fo.writeToFile(path,fileName);

	}
	
	public static void saveVector(String path, String fileName, double[] vector) {

		// System.out.println(" Saving Normalized PSSM to: " +fileName);
		DecimalFormat df = new DecimalFormat();
		df.applyPattern("#0.000000");
		StringBuffer buf = new StringBuffer();
		for (int i = 0; i < vector.length; i++) {
				buf.append(StrUtil.formatStrToFixLen(df.format(vector[i]),
						10));
		}
		// System.out.println(buf.toString());
		FileObject fo = new FileObject();
		String str = buf.toString();
		str = str.substring(0, str.length() );
		fo.setContents(str.getBytes());
		fo.writeToFile(path,fileName);

	}

	public static double [][]readMatrix(String fileName){
		Vector<double[]> PSSMVec = new Vector<double[]>();
		
		BufferedReader in = null;
		try {			
			in = new BufferedReader(new FileReader(
					fileName));
		    String line = in.readLine();

			while (line != null && line.length() == 200) {
				double [] row = new double[20];
				for(int i =0;i<20;i++){
					row[i] = Double.parseDouble(line.substring(i*10, (i+1)*10));
				}
				PSSMVec.addElement(row);
				line = in.readLine();
			}
		}catch(Exception e){
			e.printStackTrace();
		}finally{
				SocketUtil.close(in);
		}
		double [][] pssm = new double[PSSMVec.size()][20];
		for (int i = 0; i < PSSMVec.size(); i++) {
			pssm[i] = (double[]) PSSMVec.elementAt(i);
		}
		return pssm;
	}
	public void parseBlastPSSM(String blastPSSMMatrixFileName) {
		System.out.println("    Parsing Blast PSSM: " + blastPSSMMatrixFileName
				+ "...");
		Vector<int[]> originalPSSMVec = new Vector<int[]>();
		// Amino Acid Index
		// char[] AAIndex = { 'A', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'K',
		// 'L','M', 'N', 'P', 'Q', 'R', 'S', 'T', 'V', 'W', 'Y' };
		// A R N D C Q E G H I L K M F P S T W Y V

		int[] LIndex = { 1, 15, 12, 3, 2, 14, 4, 6, 7, 8, 10, 9, 11, 5, 13, 16,
				17, 19, 20, 18 };
		BufferedReader in = null;
		try {
			
			in = new BufferedReader(new FileReader(
					blastPSSMMatrixFileName));
			// Discard the first 3 lines
			in.readLine();
			in.readLine();
			in.readLine();
			String line = in.readLine();

			while (line != null && line.length() > 1) {

				line = line.substring(7, line.length());

				// Typically, AAIndex.length = 20
				int[] intArray = new int[20];

				int firstIndex = 0;
				int secondIndex = 0;

				for (int i = 0; i < LIndex.length; i++) {

					while (line.charAt(firstIndex) == ' ')
						firstIndex++;

					secondIndex = firstIndex;

					while (line.charAt(secondIndex) != ' ')
						secondIndex++;

					// System.out.print(line.substring(firstIndex,secondIndex) +
					// " ");

					int a = Integer.parseInt(line.substring(firstIndex,
							secondIndex));

					intArray[LIndex[i] - 1] = a;

					firstIndex = secondIndex;
				}
				// System.out.println();

				originalPSSMVec.add(intArray);
				line = in.readLine();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}finally{
				SocketUtil.close(in);
		}

		originalPSSM = new int[originalPSSMVec.size()][20];
		for (int i = 0; i < originalPSSMVec.size(); i++) {
			originalPSSM[i] = (int[]) originalPSSMVec.elementAt(i);
		}

		// System.out.println(" Saving Original PSSM to file:
		// /temp/originalPSSM"+ "...");
		StringBuffer buf = new StringBuffer();
		for (int i = 0; i < originalPSSM.length; i++) {
			for (int j = 0; j < originalPSSM[i].length; j++) {
				buf.append(StrUtil
						.formatStrToFixLen("" + originalPSSM[i][j], 4));
			}
			buf.append("\n");
		}
		// System.out.println(buf.toString());
		FileObject fo = new FileObject();
		String str = buf.toString();
		str = str.substring(0, str.length() -1 );
		fo.setContents(str.getBytes());
		fo.writeToFile("./temp","originalPSSM");
	}


}

class StreamGobbler extends Thread {
	InputStream is;

	String type;

	StreamGobbler(InputStream is, String type) {
		this.is = is;
		this.type = type;
	}

	public void run() {
		try {
			InputStreamReader isr = new InputStreamReader(is);
			BufferedReader br = new BufferedReader(isr);
			String line = null;
			while ((line = br.readLine()) != null) {
				if (type.equals("Error"))
					System.out.print(line+"\n");
				else {
				}
			}
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
	}
}