package edu.njust.cs.util;

import javax.swing.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import java.net.URL;
import java.net.URLClassLoader;

public class ConfigEditor extends JDialog implements ActionListener {
	private static final long serialVersionUID = 1L;
	private JButton btnOk = new JButton("ȷ(Yes)");
	private JButton btnCancel = new JButton("ȡ(Esc)");
	private JButton btnAdd = new JButton("");
	private JButton btnEdit = new JButton("༭");
	private JButton btnDelete = new JButton("ɾ");

	private CustomTableModelE modelForConfig = null;
	private JTable table = null;
	String[] columnNames = { "", "ֵ" };
	@SuppressWarnings("unchecked")
	Class [] columnTypes = { String.class, String.class };
	private URLClassLoader urlLoader = (URLClassLoader) this.getClass()
			.getClassLoader();
	private URL url = null;
	private JFrame father = null;
	private String fileName = null;

	public ConfigEditor(JFrame f, boolean b, String i_fileName) {
		super(f, b);
		this.father = f;
		this.fileName = i_fileName;
		this.setTitle(": " + i_fileName);
		this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		url = urlLoader.findResource("image/ok20.gif");
		btnOk.setIcon(new ImageIcon(url));
		url = urlLoader.findResource("image/cancel20.gif");
		btnCancel.setIcon(new ImageIcon(url));
		this.btnOk.setMnemonic(KeyEvent.VK_Y);
		this.btnCancel.setMnemonic(KeyEvent.VK_E);
		this.btnCancel.registerKeyboardAction(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				ConfigEditor.this.setVisible(false);
			}
		}, KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
				JComponent.WHEN_IN_FOCUSED_WINDOW);

		loadConfig();
		this.setSize(600, 300);
		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		this.setLocation((d.width - 600) / 2, (d.height - 300) / 2);
		this.setResizable(false);
	}

	public void loadConfig() {
		modelForConfig = new CustomTableModelE(0, columnNames.length,
				columnNames, columnTypes);

		Properties prop = new Properties();
		try {
			FileInputStream in = new FileInputStream(fileName);
			prop.load(in);
			in.close();
		} catch (Exception e) {
			JOptionPane.showMessageDialog(this, "ļʧ!\n½ã", "Ϣ",
					JOptionPane.ERROR_MESSAGE);
			e.printStackTrace();
		}
		Set <Object> keys = prop.keySet();
		for (Object o : keys) {
			String[] item = { o + "", prop.get(o) + "" };
			modelForConfig.addRow(item);
		}
		TableSorter sorter = new TableSorter(this.modelForConfig);
		table = new JTable(sorter);
		table.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				int clickCount = e.getClickCount();
				if (clickCount > 1) {
					edit();
				}
			}
		});
		sorter.addMouseListenerToHeaderInTable(table);
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		btnOk.addActionListener(this);
		btnCancel.addActionListener(this);
		this.btnAdd.addActionListener(this);
		this.btnDelete.addActionListener(this);
		this.btnEdit.addActionListener(this);
		Insets insets = new Insets(2, 2, 2, 2);
		JPanel cmdPanel = new JPanel();
		cmdPanel.setBorder(BorderFactory.createEtchedBorder());
		cmdPanel.setLayout(new GridBagLayout());
		LayoutUtil.add(cmdPanel, GridBagConstraints.NONE,
				GridBagConstraints.CENTER, 0, 0, 0, 0, 1, 1, btnAdd, insets);
		LayoutUtil.add(cmdPanel, GridBagConstraints.NONE,
				GridBagConstraints.CENTER, 0, 0, 1, 0, 1, 1, btnEdit, insets);
		LayoutUtil.add(cmdPanel, GridBagConstraints.NONE,
				GridBagConstraints.CENTER, 0, 0, 2, 0, 1, 1, btnDelete, insets);

		LayoutUtil.add(cmdPanel, GridBagConstraints.HORIZONTAL,
				GridBagConstraints.CENTER, 100, 0, 3, 0, 1, 1, new JLabel(),
				insets);
		LayoutUtil.add(cmdPanel, GridBagConstraints.NONE,
				GridBagConstraints.CENTER, 0, 0, 4, 0, 1, 1, btnOk, insets);
		LayoutUtil.add(cmdPanel, GridBagConstraints.NONE,
				GridBagConstraints.CENTER, 0, 0, 5, 0, 1, 1, btnCancel, insets);

		this.getContentPane().add(cmdPanel, BorderLayout.SOUTH);
		this.getContentPane().add(new JScrollPane(table), BorderLayout.CENTER);
		this.getRootPane().setDefaultButton(this.btnOk);

	}

	public void actionPerformed(ActionEvent e) {
		Object source = e.getSource();
		if (source == btnOk) {
			if (saveConfig()) {
				this.setVisible(false);
				JOptionPane.showMessageDialog(this, "Ч!", "ʾ",
						JOptionPane.INFORMATION_MESSAGE);
			}
		} else if (source == btnCancel) {
			this.setVisible(false);
		} else if (source == btnAdd) {
			add();
		} else if (source == btnDelete) {
			delete();

		} else if (source == btnEdit) {
			edit();
		}
	}

	public void edit() {
		int selectedRow = table.getSelectedRow();
		if (selectedRow < 0) {
			JOptionPane.showMessageDialog(this, "ȴӱѡһ!", "ʾ",
					JOptionPane.INFORMATION_MESSAGE);
			return;
		}
		String[] dataToBeEdit = {
				(String) modelForConfig.getValueAt(selectedRow, 0),
				(String) modelForConfig.getValueAt(selectedRow, 1) };
		AttrValueEditor editor = new AttrValueEditor(father, true,
				AttrValueEditor.TYPE_EDIT, dataToBeEdit);
		editor.setVisible(true);
		if (editor.action == AttrValueEditor.OK) {
			String[] dataAfterEdit = editor.getAttrAndValue();
			boolean attrAlreadyExist = false;
			for (int i = 0; i < modelForConfig.getRowCount(); i++) {
				if (i != selectedRow) {
					if (modelForConfig.getValueAt(i, 0).toString().trim()
							.equals(dataAfterEdit[0])) {
						attrAlreadyExist = true;
						break;
					}
				}
			}
			if (attrAlreadyExist) {
				JOptionPane.showMessageDialog(this, "޸ĺԺеظ,޸!",
						"ʾ", JOptionPane.INFORMATION_MESSAGE);
				return;
			}
			modelForConfig.removeRow(selectedRow);
			modelForConfig.insertRow(selectedRow, dataAfterEdit);
			table.changeSelection(selectedRow, 0, false, false);
		}
	}

	public void delete() {
		int selectedRow = table.getSelectedRow();
		if (selectedRow < 0) {
			JOptionPane.showMessageDialog(this, "ȴӱѡһ!", "ʾ",
					JOptionPane.INFORMATION_MESSAGE);
			return;
		}
		int userSelection = JOptionPane.showConfirmDialog(this, "ȷҪɾô?",
				"ȷ", JOptionPane.YES_NO_OPTION);
		if (userSelection == JOptionPane.YES_OPTION) {
			modelForConfig.removeRow(selectedRow);
		}
	}

	public void add() {
		AttrValueEditor editor = new AttrValueEditor(father, true,
				AttrValueEditor.TYPE_ADD, null);
		editor.setVisible(true);
		if (editor.action != AttrValueEditor.OK)
			return;

		String[] dataToBeAdd = editor.getAttrAndValue();
		boolean attrAlreadyExist = false;
		for (int i = 0; i < modelForConfig.getRowCount(); i++) {
			if (modelForConfig.getValueAt(i, 0).toString().trim().equals(
					dataToBeAdd[0])) {
				attrAlreadyExist = true;
				break;
			}
		}
		if (attrAlreadyExist) {
			JOptionPane.showMessageDialog(this, "Ѿ!", "ʾ",
					JOptionPane.INFORMATION_MESSAGE);
			return;
		}
		modelForConfig.addRow(dataToBeAdd);
		table
				.changeSelection(modelForConfig.getRowCount() - 1, 0, false,
						false);
	}

	public boolean saveConfig() {
		boolean saveOk = true;

		Properties prop = new Properties();
		for (int i = 0; i < modelForConfig.getRowCount(); i++) {
			prop.put(modelForConfig.getValueAt(i, 0), modelForConfig
					.getValueAt(i, 1));
		}

		try {
			FileOutputStream out = new FileOutputStream(fileName);
			prop.store(out, "Configuration Parameters");
			out.close();
		} catch (Exception e) {
			JOptionPane.showMessageDialog(this, "ļʧ!\n½ã", "Ϣ",
					JOptionPane.ERROR_MESSAGE);
			e.printStackTrace();
			saveOk = false;
		}
		return saveOk;
	}
}
