package edu.njust.cs.util;

import java.io.IOException;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.FloatControl;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.SourceDataLine;
import javax.sound.sampled.UnsupportedAudioFileException;
 
public class WavePlayer{
 
    private String filename;
 
    private Position curPosition;
 
    private final int EXTERNAL_BUFFER_SIZE = 524288; // 128Kb
 
    private boolean isAlive = false;
    
    private boolean alarmFinshed = true;
    enum Position {
        LEFT, RIGHT, NORMAL
    };
 
    public WavePlayer(String wavfile) {
        filename = wavfile;
        curPosition = Position.NORMAL;
    }
 
    public WavePlayer(String wavfile, Position p) {
        filename = wavfile;
        curPosition = p;
    }
    public synchronized boolean isAlarmFinished(){
    	return alarmFinshed;
    }
    public synchronized void setAlarmFinished(boolean b){
    	alarmFinshed = b;
    }
    public synchronized void stopAlarm(){
    	this.isAlive = false;
    }
    public synchronized boolean isAlive(){
    	return isAlive;
    }
    public synchronized void setAlive(boolean b){
    	 isAlive = b;
    }
    
    public synchronized void startAlarm(){
    	isAlive = true;
    	Thread t = new Thread(new Runnable(){
    		public void run(){
    		    alarm();
    		}
    	});
    	t.start();    	
    }
    
    private void alarm(){ 
    	alarmFinshed = false;
    	while (isAlive) {    		
			AudioInputStream audioInputStream = null;
			try {
				audioInputStream = AudioSystem
						.getAudioInputStream(WavePlayer.class
								.getResource(filename));
			} catch (UnsupportedAudioFileException e1) {
				e1.printStackTrace();
				return;
			} catch (IOException e1) {
				e1.printStackTrace();
				return;
			}

			AudioFormat format = audioInputStream.getFormat();
			SourceDataLine auline = null;
			DataLine.Info info = new DataLine.Info(SourceDataLine.class, format);

			try {
				auline = (SourceDataLine) AudioSystem.getLine(info);
				auline.open(format);
			} catch (LineUnavailableException e) {
				e.printStackTrace();
				return;
			} catch (Exception e) {
				e.printStackTrace();
				return;
			}

			if (auline.isControlSupported(FloatControl.Type.PAN)) {
				FloatControl pan = (FloatControl) auline
						.getControl(FloatControl.Type.PAN);
				if (curPosition == Position.RIGHT)
					pan.setValue(1.0f);
				else if (curPosition == Position.LEFT)
					pan.setValue(-1.0f);
			}

			auline.start();
			int nBytesRead = 0;
			byte[] abData = new byte[EXTERNAL_BUFFER_SIZE];

			try {
				while (nBytesRead != -1) {
					nBytesRead = audioInputStream
							.read(abData, 0, abData.length);
					if (nBytesRead >= 0)
						auline.write(abData, 0, nBytesRead);
				}
			} catch (IOException e) {
				e.printStackTrace();
				return;
			} finally {
				auline.drain();
				auline.close();
			}
		}
    	alarmFinshed = true;
    }
        
    public static void main(String [] args){
    
    }
}
